@@ -2,17 +2,20 @@
 '''
 #!/usr/bin/python3
 
-import os,sys,time,re,traceback
+import os,sys,time,re,traceback,platform
 from utilities.Qt import QtGui, QtCore, QtWidgets
 
 from utilities.templates import ui_layout as layout
 from utilities import dio,REGISTERS,uploader
-import constants
+import constants,inspect
 
 
 from functools import partial
 from collections import OrderedDict
 
+
+
+
 class myTimer():
 	def __init__(self,interval):
 		self.interval = interval
@@ -36,6 +39,7 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 	ports = ['A','B','C','D']
 	logThis = QtCore.pyqtSignal(str)
 	logThisPlain = QtCore.pyqtSignal(bytes)
+	serialGaugeSignal = QtCore.pyqtSignal(int)
 	def __init__(self, parent=None,**kwargs):
 		super(AppWindow, self).__init__(parent)
 		self.setupUi(self)
@@ -46,11 +50,13 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 		self.EXAMPLES_DIR = REGISTERS.VERSIONS[self.VERSION]['examples directory']
 		
 		self.docks = [self.padock,self.pbdock,self.pcdock,self.pddock]
+		self.sensorList = []
 		self.monitoring = True
 		self.logRegisters = True
 		self.userHexRunning = False
 		self.uploadingHex = False
 		self.autoUpdateUserRegisters = False
+		self.CFile = None #'~/kuttyPy.c'
 
 		self.setTheme("material")
 		examples = [a for a in os.listdir(os.path.join(path["examples"],self.EXAMPLES_DIR)) if ('.py' in a) and a is not 'kuttyPy.py'] #.py files except the library
@@ -67,7 +73,8 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 		self.codeEval.logThis.connect(self.appendLog) #Connect to the log window
 		self.logThis.connect(self.appendLog) #Connect to the log window
 		self.logThisPlain.connect(self.appendLogPlain) #Connect to the log window
-
+		self.serialGaugeSignal.connect(self.setSerialgauge)
+		
 		self.codeThread.started.connect(self.codeEval.execute)
 		self.codeThread.finished.connect(self.codeFinished)
 
@@ -107,6 +114,12 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 		'status':myTimer(constants.STATUS_UPDATE_INTERVAL),
 		'update':myTimer(constants.AUTOUPDATE_INTERVAL),
 		}
+
+		serialgaugeoptions = {'name':'Serial Monitor', 'init':print, 'read':None,
+				'fields':['Value'],
+				'min':[0],
+				'max':[255]}
+		self.serialGauge = dio.DIOSENSOR(self,serialgaugeoptions)
 		
 		self.startTime = time.time()
 		self.timer = QtCore.QTimer()
@@ -168,11 +181,12 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 			self.SR = None
 			self.GR = None
 			self.evalGlobals = {}
+
 			self.evalGlobals['getReg']  = self.getReg
 			self.evalGlobals['setReg']  = self.setReg
 			self.evalGlobals['print']  = self.printer
 
-		def setCode(self,code,SR,GR):
+		def setCode(self,code,**kwargs):
 			try:
 				self.compiled = compile(code.encode(), '<string>', mode='exec')
 			except SyntaxError as err:
@@ -186,8 +200,13 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 				cl, exc, tb = sys.exc_info()
 				line_number = traceback.extract_tb(tb)[-1][1]
 				return '''<span style="color:red;">%s at line %d: %s</span>''' % (error_class, line_number, detail)
-			self.SR = SR
-			self.GR = GR
+
+			self.SR = kwargs.get('setReg')
+			self.GR = kwargs.get('getReg')
+			self.evalGlobals = kwargs
+			self.evalGlobals['getReg']  = self.getReg #Overwrite these three. They will be wrapped.
+			self.evalGlobals['setReg']  = self.setReg
+			self.evalGlobals['print']  = self.printer
 			return ''
 			
 
@@ -240,7 +259,13 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 
 		self.log.clear() #clear the log window
 		self.log.setText('''<span style="color:green;">----------User Code Started-----------</span>''')
-		compilemsg = self.codeEval.setCode('{0:s}'.format(self.userCode.toPlainText()),self.p.setReg, self.p.getReg)
+		kwargs = {}
+		for a in dir(self.p):
+			attr = getattr(self.p, a)
+			if inspect.ismethod(attr) and a[:2]!='__':
+				kwargs[a] = attr
+
+		compilemsg = self.codeEval.setCode('{0:s}'.format(self.userCode.toPlainText()),**kwargs)
 		if len(compilemsg):
 			self.log.append(compilemsg)
 			return
@@ -250,7 +275,7 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 		self.tabs.setTabEnabled(0,False)
 
 	def codeFinished(self):
-		print ('hello')
+		print('finished')
 		self.tabs.setTabEnabled(0,True)
 		self.userCode.setStyleSheet("")
 		self.uploadingHex = False
@@ -281,6 +306,9 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 		self.log.moveCursor(QtGui.QTextCursor.End)
 		self.log.insertPlainText(txt.decode('ascii'))
 
+	def setSerialgauge(self,val):
+		self.serialGauge.setValue([val])
+
 	def genLog(self):
 		html='''<table border="1" align="center" cellpadding="1" cellspacing="0" style="font-family:arial,helvetica,sans-serif;font-size:9pt;">
 		<tbody><tr><td colspan="4">%s</td></tr>'''%(time.ctime())
@@ -314,6 +342,7 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 		if self.userHexRunning:
 			t = self.p.fd.read(self.p.fd.in_waiting)
 			if len(t):
+				self.serialGaugeSignal.emit(t[0])
 				self.logThisPlain.emit(t)
 			return
 		
@@ -348,18 +377,22 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 			elif a[0] == 'WRITE': #['WRITE','REGNAME',val]
 				self.setReg(a[1],a[2]);
 			elif a[0] == 'READ': #['READ','REGNAME',function]
-				val = self.getReg(a[1]);
+				val = self.getReg(a[1])
 				a[2](val)
 			elif a[0] == 'CNTR1': #['CNTR1',output with setValue function]
-				cl = self.getReg('TCNT1L',False);
-				ch = self.getReg('TCNT1H',False);
+				cl = self.getReg('TCNT1L',False)
+				ch = self.getReg('TCNT1H',False)
 				a[1].setValue(cl|(ch<<8))
 			elif a[0] == 'ADC': #['ADC',ADMUX,output with setValue function, to log, or not to log]
-				self.setReg('ADMUX',a[1],a[3]);
-				self.setReg('ADCSRA',196|1,a[3]);
-				adcl = self.getReg('ADCL',a[3]);
-				adch = self.getReg('ADCH',a[3]);
+				self.setReg('ADMUX',a[1],a[3])
+				self.setReg('ADCSRA',196|1,a[3])
+				adcl = self.getReg('ADCL',a[3])
+				adch = self.getReg('ADCH',a[3])
 				a[2].setValue(adcl|(adch<<8))
+		
+		for a in self.sensorList:
+			if a[0].isVisible():
+				a[0].setValue(a[0].read())
 			
 		if self.enableLog.isChecked():
 			if self.clearLog.isChecked() and len(self.updatedRegs):
@@ -389,10 +422,48 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 			btn.nameIn.setChecked((value>>a)&1)
 			if portchar == 'A': #ADC
 				if btn.currentPage == 2: #ADC Page displayed
-					self.commandQ.append(['ADC',btn.ADMUX,btn.slider,btn.logstate])
+					self.commandQ.append(['ADC',btn.ADMUX,btn,btn.logstate])
 			elif type(btn)==dio.DIOCNTR and btn.currentPage==2: # CNTR
 					self.commandQ.append(['CNTR1',btn.slider])
 
+	############ I2C SENSORS #################
+	def I2CScan(self):
+		if self.p.connected:
+			x = self.p.I2CScan()
+			print('Responses from: ',x)
+			for a in self.sensorList:
+				a[0].setParent(None)
+				a[1].setParent(None)
+			self.sensorList = []
+			for a in x:
+				s = self.p.sensors.get(a,None)
+				if s is not None:
+					btn = QtWidgets.QPushButton(s['name']+':'+hex(a))
+					dialog = dio.DIOSENSOR(self,s)
+					btn.clicked.connect(dialog.launch)
+					self.sensorLayout.addWidget(btn)
+					self.sensorList.append([dialog,btn])
+					continue
+
+				s = self.p.controllers.get(a,None)
+				if s is not None:
+					btn = QtWidgets.QPushButton(s['name']+':'+hex(a))
+					dialog = dio.DIOCONTROL(self,s)
+					btn.clicked.connect(dialog.launch)
+					self.sensorLayout.addWidget(btn)
+					#self.sensorList.append([dialog,btn])
+					continue
+
+				s = self.p.special.get(a,None)
+				if s is not None:
+					btn = QtWidgets.QPushButton(s['name']+':'+hex(a))
+					dialog = dio.DIOROBOT(self,s)
+					btn.clicked.connect(dialog.launch)
+					self.sensorLayout.addWidget(btn)
+					#self.sensorList.append([dialog,btn])
+					continue
+
+
 	def loadExample(self,filename):
 		self.userCode.setPlainText(open(os.path.join(path["examples"],self.EXAMPLES_DIR,filename), "r").read())
 
@@ -413,28 +484,47 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 			self.mode = mode
 
 		def execute(self):
+			if self.mode == 'compileupload':
+				try:
+					import subprocess
+					fname = self.fname.split(".")[0]
+					cmd = 'avr-gcc -Wall -O2 -mmcu=%s -o %s %s' %('atmega32',fname,self.fname)
+					res = subprocess.getstatusoutput(cmd)
+					if res[0] != 0:
+						self.logThis.emit('''<span style="color:red;">Compile Error: %s</span>'''%res[1])
+						self.finished.emit()
+						return
+
+					else:
+						self.logThis.emit('''<span style="color:white;">%s</span><br>'''%res[1])
+					cmd = 'avr-objcopy -j .text -j .data -O ihex %s %s.hex' %(fname,fname)
+					res = subprocess.getstatusoutput(cmd)
+					self.logThis.emit('''<span style="color:white;">%s</span><br>'''%res[1])
+					cmd = 'avr-objdump -S %s > %s.lst'%(fname,fname)
+					res = subprocess.getstatusoutput(cmd)
+					self.logThis.emit('''<span style="color:white;">%s</span><br>'''%res[1])
+					if self.fname[-2:] in ['.c','.C']:
+						self.fname = self.fname[:-2]+'.hex' #Replace .c with .hex
+						self.mode = 'upload'
+						self.logThis.emit('''<span style="color:green;">Generated Hex File</span>''')
+					self.logThis.emit('''<span style="color:green;">Finished Compiling: Generated Hex File</span>''')
+				except Exception as err:
+					self.logThis.emit('''<span style="color:red;">Failed to Compile:%s</span>'''%str(err))
+
 			if self.p.connected:
 				if self.mode == 'upload':
 					try:
-						self.p.fd.setRTS(0);time.sleep(0.01);self.p.fd.setRTS(1);time.sleep(0.2)
+						self.p.fd.setRTS(0);time.sleep(0.01);self.p.fd.setRTS(1);time.sleep(0.4)
 						dude = uploader.Uploader(self.p.fd, hexfile=self.fname,logger = self.logThis)
 						dude.program()
 						dude.verify()
+						self.p.fd.setRTS(0);time.sleep(0.01);self.p.fd.setRTS(1);time.sleep(0.2)
+						self.p.get_version()
+						self.logThis.emit('''<span style="color:green;">Finished upload</span>''')
 					except Exception as err:
 						self.logThis.emit('''<span style="color:red;">Failed to upload</span>''')
-					self.p.fd.setRTS(0);time.sleep(0.01);self.p.fd.setRTS(1);time.sleep(0.2)
-					self.logThis.emit('''<span style="color:green;">Finished upload</span>''')
-				'''
-				elif self.mode == 'monitor':
-					while self.mode == 'monitor':
-						t = self.p.fd.readline()
-						if len(t):
-							self.logThisPlain.emit(t)
-				'''
 			self.finished.emit()
 	
-
-
 	def uploadHex(self):
 		filename = QtWidgets.QFileDialog.getOpenFileName(self," Open a hex file to upload to your KuttyPy", "", "Hex Files (*.hex)")
 		if len(filename[0]):
@@ -447,6 +537,27 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 			self.uploadThread.start()
 
 
+	def openFile(self):
+		filename = QtWidgets.QFileDialog.getOpenFileName(self," Open a C file to edit", path["examples"], "C Files (*.c *.C)")
+		if len(filename[0]):
+			self.filenameLabel.setText(filename[0])
+			self.CFile = filename[0]
+			self.log.clear()
+			self.log.setText('''<span style="color:cyan;">-- Opened File: %s --</span><br>'''%filename[0])
+			if 'inux' in platform.system(): #Linux based system
+				os.system('%s %s' % ('xdg-open', filename[0]))
+			else:
+				os.system('%s %s' % ('open', filename[0]))
+
+	def compileAndUpload(self):
+		if self.CFile:
+			self.uploadingHex = True
+			self.log.clear()
+			self.log.setText('''<span style="color:cyan;">-- Compiling and Uploading Code --</span><br>''')
+			self.UploadObject.config('compileupload',self.p,self.CFile)
+			self.uploadThread.start()
+			
+
 	##############################
 	def setTheme(self,theme):
 		self.setStyleSheet("")
@@ -478,6 +589,7 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 				self.tabs.setEnabled(False)
 				self.log.clear()
 				self.log.setText('''<span style="color:cyan;">-- Serial Port Monitor --</span><br>''')
+				self.serialGauge.show()
 
 			else:
 				self.p.fd.setRTS(0)  #Trigger a reset
@@ -486,10 +598,12 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 				time.sleep(0.1)
 				while self.p.fd.in_waiting:
 					self.p.fd.read()
+				self.p.get_version()
 				for a in self.docks:
 					a.setEnabled(True)
 				self.userHexRunning=False
-				self.tabs.setEnabled(True)		
+				self.tabs.setEnabled(True)
+				self.serialGauge.hide()
 		else:
 			if self.isChecked():
 				self.setChecked(False)
